     /*
 DSPRobotics
 PIC Clicker Accel Clcik - HID  V1.00    (PIC Clicker)
 Language: Mikro C Pro
 Chip: P18F47J53
 CLK: 48Mhz

  HID Data:
 Char 0 Read:
 bit 0 = LED1
 bit 1 = LED2

 Char 0 Write:
 bit 0 = Button1
 bit 1 = Button2
 Chars 1 - 10 = X
 Chars 11- 20 = Y
 Chars 21- 30 = Z

 Aplication: Ideal for using with the FlowStone Graphical Programming Language.
 www.dsprobotics.com

 */
/*---------------------------------------------------------------*/
// ADXL345 Register Definition
#define _POWER_CTL      0x2D
#define _DATA_FORMAT    0x31
#define _BW_RATE        0x2C
#define _DATAX0         0x32
#define _DATAX1         0x33
#define _DATAY0         0x34
#define _DATAY1         0x35
#define _DATAZ0         0x36
#define _DATAZ1         0x37
#define _FIFO_CTL       0x38
#define _SPEED          0x0F         // Buffer Speed - 3200Hz

#define _ACCEL_ERROR    0x02

#define _ACCEL_ADDRESS  0x3A         //alt A6  // 3A

// USB buffers
unsigned char readbuff[64] absolute 0x500;   // Buffers should be in USB RAM, please consult datasheet
unsigned char writebuff[64] absolute 0x540;
//General
char kk;
char i;
// pin definitions
//LEDs
sbit LD1 at LATA0_bit;    //LED 1
sbit LD2 at LATA1_bit;    //LED 2
sbit LD1_Direction at TRISA0_bit;
sbit LD2_Direction at TRISA1_bit;
//Button Inputs
sbit T1 at RD3_bit;   //Button 1
sbit T2 at RD2_bit;   //Button 2
sbit T1_Direction at TRISD3_bit;
sbit T2_Direction at TRISD2_bit;
//Accel
sbit ACCEL_CS at RB3_bit;
sbit ACCEL_CS_Dir at TRISB3_bit;

unsigned short temp;
char out[16];
int readings[3] = {0, 0, 0}; // X, Y and Z
short address, buffer;
/*---------------------------------------------------------------*/
// Interrupt routine
void interrupt() {

 USB_Interrupt_Proc();                   // USB servicing is done inside the interrupt
}
/*---------------------------------------------------------------*/
void clear_USB_buffers()
{
   for (i=0; i<63;i++)
   {
   readbuff[i] = 0;
   writebuff[i] = 0;
   }
}
/*---------------------------------------------------------------*/
char read_button1() {
  if (Button(&PORTD, 3, 2, 0)) {               // Detect logical one
      return  1;                              // Update flag
    } else return 0;
}
/*---------------------------------------------------------------*/
char read_button2() {
  if (Button(&PORTD, 2, 2, 0)) {               // Detect logical one
      return  2;                              // Update flag
    } else return 0;
}
/*---------------------------------------------------------------*/
void ADXL345_Write(unsigned short address, unsigned short data1) {
  I2C2_Start();              // issue I2C start signal
  I2C2_Wr(_ACCEL_ADDRESS);   // send byte via I2C  (device address + W)
  I2C2_Wr(address);          // send byte (address of the location)
  I2C2_Wr(data1);            // send data (data to be written)
  I2C2_Stop();               // issue I2C stop signal
}
/*---------------------------------------------------------------*/
unsigned short ADXL345_Read(unsigned short address) {
  unsigned short tmp = 0;

  I2C2_Start();              // issue I2C start signal
  I2C2_Wr(_ACCEL_ADDRESS);   // send byte via I2C (device address + W)
  I2C2_Wr(address);          // send byte (data address)

  I2C2_Start();              // issue I2C signal repeated start
  I2C2_Wr(_ACCEL_ADDRESS+1); // send byte (device address + R)
  tmp = I2C2_Rd(0);          // Read the data (NO acknowledge)
  I2C2_Stop();               // issue I2C stop signal

  return tmp;
}
/*---------------------------------------------------------------*/
char ADXL345_Init(void) {
  char id = 0x00;

  // Go into standby mode to configure the device.
  ADXL345_Write(0x2D, 0x00);

  id = ADXL345_Read(0x00);
  if (id != 0xE5) {
    return _ACCEL_ERROR;
  }
  else {
    ADXL345_Write(_DATA_FORMAT, 0x08);       // Full resolution, +/-2g, 4mg/LSB, right justified
    ADXL345_Write(_BW_RATE, 0x0A);           // Set 100 Hz data rate
    ADXL345_Write(_FIFO_CTL, 0x80);          // stream mode
    ADXL345_Write(_POWER_CTL, 0x08);         // POWER_CTL reg: measurement mode
    return 0x00;
  }
}
/*---------------------------------------------------------------*/
// Read X Axis
int Accel_ReadX(void) {
  char low_byte;
  int Out_x;

  Out_x = ADXL345_Read(_DATAX1);
  low_byte = ADXL345_Read(_DATAX0);

  Out_x = (Out_x << 8);
  Out_x = (Out_x | low_byte);

  return Out_x;
}
/*---------------------------------------------------------------*/
// Read Y Axis
int Accel_ReadY(void) {
  char low_byte;
  int Out_y;

  Out_y = ADXL345_Read(_DATAY1);
  low_byte = ADXL345_Read(_DATAY0);

  Out_y = (Out_y << 8);
  Out_y = (Out_y | low_byte);

  return Out_y;
}
/*---------------------------------------------------------------*/
// Read Z Axis
int Accel_ReadZ(void) {
  char low_byte;
  int Out_z;

  Out_z = ADXL345_Read(_DATAZ1);
  low_byte = ADXL345_Read(_DATAZ0);

  Out_z = (Out_z << 8);
  Out_z = (Out_z | low_byte);

  return Out_z;
}
/*---------------------------------------------------------------*/
// Calculate the average values of the X, Y and Z axis reads
void Accel_Average(void) {
  int i, sx, sy, sz;

  // sum
  sx = sy = sz = 0;

  // average accelerometer reading over last 16 samples
  for (i=0; i<16; i++) {
    sx += Accel_ReadX();
    sy += Accel_ReadY();
    sz += Accel_ReadZ();
  }
  // average
  readings[0] = sx >> 4;
  readings[1] = sy >> 4;
  readings[2] = sz >> 4;

}
/*---------------------------------------------------------------*/
void AccelInit(){
  // Initialize ADXL345 accelerometer
  if (ADXL345_Init() == 0) {
   //OK
    Delay_ms(2000);
  }
  else {
    //Error
    Delay_ms(2000);
  }
}
/*---------------------------------------------------------------*/
void load_X() {
//Writes Temp text to USB buffer
 for (i =0;i <10;i++) {
    writebuff[1+i]= out[i];
 }
}
/*---------------------------------------------------------------*/
void load_Y() {
//Writes Temp text to USB buffer
 for (i =0;i <10;i++) {
    writebuff[11+i]= out[i];
 }
}
/*---------------------------------------------------------------*/
void load_Z() {
//Writes Temp text to USB buffer
 for (i =0;i <10;i++) {
    writebuff[21+i]= out[i];
 }
}
/*---------------------------------------------------------------*/
void main(){
  long cnt = 0;

  ANCON0 = 0xFF;               // Default all pins to digital
  ANCON1 = 0x1F;               //
  TRISD = 0;
  TRISB = 0;
  TRISA = 0;
  RCSTA1 = 1;

  //Buttons
  T1_Direction = 1;         // Set direction for buttons
  T2_Direction = 1;
  //Leds
  LD1_Direction = 0;        // Set direction for LEDS
  LD2_Direction = 0;
  LD1 = 0;
  LD2 = 0;                 // turn off leds
  //Accel
  ACCEL_CS_Dir = 0;
  ACCEL_CS = 1;
  //I2C
  I2C2_Init(50000);                     // performs I2C initialization
  Delay_ms(1000);
  AccelInit();
  Delay_ms(150);

 //HID
  clear_USB_buffers();
  HID_Enable(&readbuff,&writebuff);       // Enable HID communication
 /*---------------------------------------------------------------*/
 //Main Loop
  while (1){
  Accel_Average();               // Calculate average X, Y and Z reads
  //USB
  kk = HID_Read();
  if (kk != 0)
  {
  //Accel
   IntToStr(readings[0], out);
   load_X();                            //Loads Temp into HID Buffer
   IntToStr(readings[1], out);
   load_Y();                            //Loads Temp into HID Buffer
   IntToStr(readings[2], out);
   load_Z();                            //Loads Temp into HID Buffer
   //LEDS
   if ((readbuff[0] & 0x01 )> 0) {
     LD1 = 1;
   }  else LD1 = 0;
   if ((readbuff[0] & 0x02 )> 0) {
     LD2 = 1;
   }  else LD2 = 0;
   //Buttons
   writebuff[0]=0;
   writebuff[0] = writebuff[0] | read_button1() ;
   writebuff[0] = writebuff[0] | read_button2() ;
   //HID
   HID_Write(&writebuff,64);
  }
 }
}